#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.icons.font_awesome
      @file: form_icons.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from hspylib.modules.cli.icons.font_awesome.awesome import Awesome


class FormIcons(Awesome):
    """
        Form UI icons.
        Codes can be found here:
        - https://fontawesome.com/cheatsheet?from=io
    """

    # @formatter:off
    ARROW_LEFT = u'\uF060'     # 
    ARROW_RIGHT = u'\uF061'    # 
    ARROW_UP = u'\uF062'       # 
    ARROW_DOWN = u'\uF063'     # 
    SELECTOR = u'\uFC32'       # ﰲ

    CHECK = u'\uF00C'          # 
    ERROR = u'\uf057'          # 
    CHECK_CIRCLE = u'\uF058'   # 
    UNCHECK_CIRCLE = u'\uF111' # 
    CHECK_SQUARE = u'\uF14A'   # 
    UNCHECK_SQUARE = u'\uF0C8' # 
    ON = u'\uF205'             # 
    OFF = u'\uF204'            # 
    MARKED = u'\uF634'         # 
    UNMARKED = u'\uF630'       # 
    CLEAR = u'\uF5E1'          # 
    REFRESH = u'\uF01E'        # 

    HIDDEN = u'\uF070'         # 
    VISIBLE = u'\uF06E'        # 
    LOCKED = u'\uF023'         # 
    UNLOCKED = u'\uF09C'       # 
    EDITABLE = u'\uF044'       # 
    MASKED = u'\uF0CE'         # 
    SELECTABLE = u'\uF150'     # 
    DESELECT = u'\uF657'       # 
    DELETE = u'\uF014'         # 
    EDIT = u'\uF040'           # 
    PLUS = u'\uF067'           # 
    MINUS = u'\uF068'          # 
    # @formatter:on



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.core.tools
      @file: text_tools.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import random
import re
from abc import ABC

from hspylib.core.tools.commons import get_or_default


def fit_text(text: str, width: int) -> str:
    """TODO"""
    return text if len(text) <= width else text[0:width - 3] + '...'


def rand_string(choices: str, length: int) -> str:
    """TODO"""
    return ''.join(random.choices(choices, k=length))


def justified_left(string: str, width: int, fill: str = ' ') -> str:
    """TODO"""
    return string.ljust(width, fill)


def justified_center(string: str, width: int, fill: str = ' ') -> str:
    """TODO"""
    return string.center(width, fill)


def justified_right(string: str, width: int, fill: str = ' ') -> str:
    """TODO"""
    return string.rjust(width, fill)


def uppercase(string: str) -> str:
    """TODO"""
    return string.upper()


def lowercase(string: str) -> str:
    """TODO"""
    return string.lower()


def camelcase(string: str, separator: str = '_|-') -> str:
    """TODO"""
    parts = re.split(rf'{separator}+', string)
    return '_'.join([p.capitalize() for p in parts])


def snakecase(string: str) -> str:
    """TODO"""
    return string.strip().lower().replace(' ', '_').replace('-', '_')


def cut(string: str, index: int, separator: str = ' ') -> str:
    """TODO"""
    result = tuple(re.split(rf'{separator}+', string))
    return get_or_default(result, index)


def strip_escapes(string: str) -> str:
    """TODO"""
    return re.compile(r'\x1b[^m]*m').sub('', string)


# pylint: disable=too-few-public-methods
class TextAlignment(ABC):
    """
    Table cell text justification helper.
    """
    LEFT = justified_left
    CENTER = justified_center
    RIGHT = justified_right


# pylint: disable=too-few-public-methods
class TextCase(ABC):
    """
    Table cell text justification helper.
    """
    UPPER_CASE = uppercase
    LOWER_CASE = lowercase
    CAMEL_CASE = camelcase



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.vt100
      @file: terminal.py
   @created: Tue, 11 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import logging as log
import select
import shlex
import subprocess
from abc import ABC
from typing import Optional

from hspylib.core.tools.commons import syserr


class Terminal(ABC):
    """TODO"""

    @staticmethod
    def shell_exec(cmd_line: str, **kwargs) -> Optional[str]:
        """TODO"""
        try:
            log.info("Executing shell command: %s", cmd_line)
            cmd_args = list(filter(None, shlex.split(cmd_line)))
            result = subprocess.check_output(cmd_args, **kwargs).decode("utf-8")
            log.info("Execution result: %s", result)
            return result.strip() if result else None
        except subprocess.CalledProcessError as err:
            log.error("Failed => %s", err)
            syserr(str(err))
            return None

    @staticmethod
    def shell_poll(cmd_line: str, **kwargs) -> None:
        """TODO"""
        if 'stdout' in kwargs:
            del kwargs['stdout']  # Deleted since we use our own
        if 'stderr' in kwargs:
            del kwargs['stderr']  # Deleted since we use our own
        try:
            log.info("Polling shell command: %s", cmd_line)
            cmd_args = list(filter(None, shlex.split(cmd_line)))
            with(subprocess.Popen(cmd_args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kwargs)) as file:
                process = select.poll()
                process.register(file.stdout)
                while True:
                    line = bytes(file.stdout.readline()).decode('utf-8').strip()
                    print('.' + line)
        except (InterruptedError, KeyboardInterrupt):
            log.warning("Polling process has been interrupted command='%s'", cmd_line)
        except subprocess.CalledProcessError as err:
            log.debug("Failed => %s", err)
            syserr(str(err))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @Package: hspylib.main.hspylib.core.crud.db.firebase
      @file: firebase_repository.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import json
import logging as log
import uuid
from abc import abstractmethod
from typing import Optional

from requests.exceptions import HTTPError
from requests.structures import CaseInsensitiveDict

from hspylib.core.crud.crud_entity import CrudEntity
from hspylib.core.crud.crud_repository import CrudRepository
from hspylib.core.crud.db.firebase.firebase_config import FirebaseConfig
from hspylib.core.enums.http_code import HttpCode
from hspylib.core.tools.preconditions import check_not_none, check_state
from hspylib.modules.fetch.fetch import delete, get, put


class FirebaseRepository(CrudRepository):
    """TODO"""

    def __init__(self):
        self.payload = None
        self.config = FirebaseConfig()

    def __str__(self):
        return str(self.payload)

    def to_list(self, json_string: str, filters: CaseInsensitiveDict = None) -> list:
        """TODO"""
        ret_list = []
        for value in json.loads(json_string).values():
            if not filters or all(k in value and value[k] == v for k, v in filters.items()):
                ret_list.append(self.row_to_entity(value))
        return ret_list

    def insert(self, entity: CrudEntity) -> None:
        """TODO"""
        entity.uuid = entity.uuid if entity.uuid is not None else str(uuid.uuid4())
        url = '{}/{}.json'.format(self.config.base_url(), entity.uuid)
        payload = entity.to_json()
        log.debug("Inserting firebase entry: %s into: %s", entity, url)
        response = put(url, payload)
        check_not_none(response, "Response is none")
        if response.status_code != HttpCode.OK:
            raise HTTPError('{} - Unable to put into={} with json_string={}'.format(response.status_code, url, payload))

    def update(self, entity: CrudEntity) -> None:
        """TODO"""
        url = '{}/{}.json'.format(self.config.base_url(), entity.uuid)
        payload = entity.to_json()
        log.debug('Updating firebase entry: %s into: %s', entity, url)
        response = put(url, payload)
        check_not_none(response, "Response is none")
        if response.status_code != HttpCode.OK:
            raise HTTPError('{} - Unable to put into={} with json_string={}'.format(response.status_code, url, payload))

    def delete(self, entity: CrudEntity) -> None:
        """TODO"""
        url = '{}/{}.json'.format(self.config.base_url(), entity.uuid)
        log.debug('Deleting firebase entry: %s into: %s', entity, url)
        response = delete(url)
        check_not_none(response, "Response is none")
        if response.status_code != HttpCode.OK:
            raise HTTPError('{} - Unable to delete from={}'.format(response.status_code, url))

    def find_all(self, filters: CaseInsensitiveDict = None) -> Optional[list]:
        """TODO"""
        url = '{}.json?orderBy="$key"'.format(self.config.base_url())
        log.debug('Fetching firebase entries from %s', url)
        response = get(url)
        check_not_none(response, "Response is none")
        if response.status_code != HttpCode.OK:
            raise HTTPError('{} - Unable to get from={}'.format(response.status_code, url))

        return self.to_list(response.body, filters) if response.body else []

    def find_by_id(self, entity_id: str) -> Optional[CrudEntity]:
        """TODO"""
        url = '{}.json?orderBy="$key"&equalTo="{}"'.format(self.config.base_url(), entity_id)
        log.debug('Fetching firebase entry entity_id=%s from %s', entity_id, url)
        response = get(url)
        check_not_none(response, "Response is none")
        if response.status_code != HttpCode.OK:
            raise HTTPError('{} - Unable to get from={}'.format(response.status_code, url))
        result = self.to_list(response.body) if response.body else []
        check_state(len(result) <= 1, "Multiple results found with entity_id={}", entity_id)

        return result[0] if len(result) > 0 else None

    @abstractmethod
    def row_to_entity(self, row: dict) -> CrudEntity:
        pass

    @abstractmethod
    def database_name(self) -> str:
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.eventbus
      @file: eventbus.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from typing import Any, Callable, List

from hspylib.core.tools.commons import syserr
from hspylib.modules.eventbus.event import Event


class EventBus:
    """TODO"""

    _buses = {}
    _subscribers = {}
    _events: List[Event] = list()

    @classmethod
    def get(cls, bus_name: str) -> Any:
        """TODO"""
        if bus_name in cls._buses:
            return cls._buses[bus_name]
        bus_instance = EventBus(bus_name)
        cls._buses[bus_name] = bus_instance
        return bus_instance

    @classmethod
    def _get_subscriber(cls, bus_name: str, event_name: str) -> Any:
        """TODO"""
        cache_key = '{}.{}'.format(bus_name, event_name)
        if cache_key in cls._subscribers:
            return cls._subscribers[cache_key]
        subscriber = {'callbacks': []}
        cls._subscribers[cache_key] = subscriber
        return subscriber

    def __init__(self, name: str):
        self.name = name

    def subscribe(self, event_name: str, cb_event_handler: Callable) -> None:
        """TODO"""
        subscriber = self._get_subscriber(self.name, event_name)
        subscriber['callbacks'].append(cb_event_handler)

    def emit(self, event_name: str, **kwargs) -> None:
        """TODO"""
        self._events.append(Event(event_name, **kwargs))
        while len(self._events) > 0:
            event = self._events.pop()
            cache_key = f"{self.name}.{event.name}"
            subscribers = self._subscribers[cache_key] if cache_key in self._subscribers else None
            if subscribers and len(subscribers['callbacks']) > 0:
                for callback in subscribers['callbacks']:
                    try:
                        callback(event)
                    except TypeError as err:
                        syserr(f"{self.__class__.__name__}::emit Callback invocation failed - {str(err)}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @Package: hspylib.main.hspylib.core.enum
      @file: content_type.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from hspylib.core.enums.enumeration import Enumeration


class ContentType(Enumeration):
    """TODO"""

    # @formatter:off
    APPLICATION_ATOM_XML                = 'application/atom+xml'
    APPLICATION_ECMASCRIPT              = 'application/ecmascript'
    APPLICATION_JSON                    = 'application/json'
    APPLICATION_VNP_API_JSON            = 'application/vpn.api+json'
    APPLICATION_JAVASCRIPT              = 'application/javascript'
    APPLICATION_OCTET_STREAM            = 'application/octet-stream'
    APPLICATION_OGG                     = 'application/ogg'
    APPLICATION_PDF                     = 'application/pdf'
    APPLICATION_POSTSCRIPT              = 'application/postscript'
    APPLICATION_RDF_XML                 = 'application/rdf+xml'
    APPLICATION_RSS_XML                 = 'application/rss+xml'
    APPLICATION_SOAP_XML                = 'application/soap+xml'
    APPLICATION_FONT_WOFF               = 'application/font-woff'
    APPLICATION_X_YAML                  = 'application/x-yaml'
    APPLICATION_XHTML_XML               = 'application/xhtml+xml'
    APPLICATION_XML                     = 'application/xml'
    APPLICATION_DTD                     = 'application/xml-dtd'
    APPLICATION_XOP_XML                 = 'application/xop+xml'
    APPLICATION_ZIP                     = 'application/zip'
    APPLICATION_GZIP                    = 'application/gzip'
    APPLICATION_GRAPHQL                 = 'application/graphql'
    APPLICATION_X_WWW_FORM_URLENCODED   = 'application/x-www-form-urlencoded'
    AUDIO_BASIC                         = 'audio/basic'
    AUDIO_L24                           = 'audio/L24'
    AUDIO_MP4                           = 'audio/mp4'
    AUDIO_MPEG                          = 'audio/mpeg'
    AUDIO_OGG                           = 'audio/ogg'
    AUDIO_VORBIX                        = 'audio/vorbis'
    AUDIO_VND_RN_REALAUDIO              = 'audio/vnd.rn-realaudio'
    AUDIO_VND_WAVE                      = 'audio/vnd.wave'
    AUDIO_WEBM                          = 'audio/webm'
    IMAGE_GIF                           = 'image/gif'
    IMAGE_JPEG                          = 'image/jpeg'
    IMAGE_PJPEG                         = 'image/pjpeg'
    IMAGE_PNG                           = 'image/png'
    IMAGE_SVG_XML                       = 'image/svg+xml'
    IMAGE_TIFF                          = 'image/tiff'
    MESSAGE_HTTP                        = 'prompt_msg/http'
    MESSAGE_IMDN_XML                    = 'prompt_msg/imdn+xml'
    MESSAGE_PARTIAL                     = 'prompt_msg/partial'
    MESSAGE_RFC_822                     = 'prompt_msg/rfc822'
    MULTIPART_MIXED                     = 'multipart/mixed'
    MULTIPART_ALTERNATIVE               = 'multipart/alternative'
    MULTIPART_RELATED                   = 'multipart/related'
    MULTIPART_FORM_DATA                 = 'multipart/form-data'
    MULTIPART_SIGNED                    = 'multipart/signed'
    MULTIPART_ENCRYPTED                 = 'multipart/encrypted'
    TEXT_CMD                            = 'text/cmd'
    TEXT_CSS                            = 'text/css'
    TEXT_CSV                            = 'text/cvs'
    TEXT_HTML                           = 'text/html'
    TEXT_PLAIN                          = 'text/plain'
    TEXT_VCARD                          = 'text/vcard'
    TEXT_XML                            = 'text/xml'
    # @formatter:on



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.qt.promotions
      @file: hconsole.py
   @created: Wed, 30 Jun 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from typing import Optional

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont, QColor, QTextCursor, QCursor
from PyQt5.QtWidgets import QWidget, QTextBrowser


class HConsole(QTextBrowser):
    """TODO"""

    def __init__(self, parent: Optional[QWidget], max_lines: int = 1000):
        super().__init__(parent)
        self._context_menu_enable = True
        self._clearable = True
        self._max_lines = max_lines
        self.setPlaceholderText('No messages received yet')
        self.setReadOnly(True)
        self.setFont(QFont("Courier New", 14))
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self._context_menu)

    def line_count(self) -> int:
        """Return the number of lines the console contains"""
        return self.document().blockCount()

    def is_empty(self) -> bool:
        """Return true if console has no lines"""
        return self.line_count() == 0

    def push_text(self, text: str, color: QColor = None) -> None:
        """Push text to the console. If the maximum buffer size reached,
           the first lines are erased
        """
        fmt_text = f"<font color={color.name() if color else '#FFFFFF'}>{text}</font>"
        if self.line_count() + 1 > self._max_lines:
            self.pop_text()
        self.append(fmt_text)

    def pop_text(self, count: int = 0) -> str:
        """Pop <count> lines form the top->bottom"""
        cursor = self.textCursor()
        cursor.movePosition(QTextCursor.Start)
        cursor.movePosition(QTextCursor.Down, QTextCursor.MoveAnchor, count)
        cursor.select(QTextCursor.LineUnderCursor)
        selected_text = cursor.selectedText()
        cursor.removeSelectedText()
        cursor.deleteChar()
        self.setTextCursor(cursor)

        return selected_text

    def _context_menu(self) -> None:
        """Display the custom context menu"""
        if self._context_menu_enable:
            ctx_menu = self.createStandardContextMenu()
            if self._clearable:
                ctx_menu.addSeparator()
                ctx_menu.addAction(u'Clear', self.clear)
            ctx_menu.exec_(QCursor.pos())

    def set_context_menu_enable(self, enabled: bool = True):
        """Whether context menu is enabled or not"""
        self._context_menu_enable = enabled

    def set_clearable(self, clearable: bool = True):
        """Whether the widget is clearable or not"""
        self._clearable = clearable



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @Package: hspylib.main.hspylib.core.config
      @file: app_config.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import logging as log
import os
from typing import Any, Optional

from hspylib.core.config.properties import Properties
from hspylib.core.metaclass.singleton import Singleton
from hspylib.core.tools.commons import dirname, log_init
from hspylib.core.tools.preconditions import check_argument, check_state


class AppConfigs(metaclass=Singleton):
    """Holds all of the application configurations (properties)"""

    DISPLAY_FORMAT = """
AppConfigs
  |-SourceDir = {}
  |-ResourceDir = {}
  |-LogFile = {}
  |-AppProperties:
   \\-{}
"""

    def __init__(
        self,
        source_root: str = None,
        resource_dir: str = None,
        log_dir: str = None,
        log_file: str = None):
        self._source_dir = source_root \
            if source_root else os.environ.get('SOURCE_ROOT', dirname(__file__))
        check_argument(os.path.exists(self._source_dir), "Unable to find the source dir: {}", self._source_dir)

        self._resource_dir = resource_dir \
            if resource_dir else os.environ.get('RESOURCE_DIR', f"{self._source_dir}/resources")
        check_argument(os.path.exists(self._resource_dir), "Unable to find the resources dir: {}", self._resource_dir)

        self._log_dir = log_dir \
            if log_dir else os.environ.get('LOG_DIR', f"{self._resource_dir}/log")
        check_argument(os.path.exists(self._log_dir), "Unable to find the log dir: {}", self._log_dir)

        self._log_file = log_file \
            if log_file else f"{self._log_dir}/application.log"
        check_state(log_init(self._log_file), "Unable to create logger: {}", self._log_file)

        self._app_properties = Properties(load_dir=self._resource_dir)
        log.info(self)

    def __str__(self):
        return '\n{}{}{}'.format(
            '-=' * 40,
            self.DISPLAY_FORMAT.format(
                str(self._source_dir),
                str(self._resource_dir),
                str(self._log_file),
                str(self._app_properties).replace('\n', '\n   |-')
                if len(self._app_properties) > 0 else ''
            ),
            '-=' * 40
        )

    def __repr__(self):
        return str(self)

    def __getitem__(self, item: str) -> Any:
        return self.get(item)

    def __len__(self) -> int:
        return len(self._app_properties)

    def source_dir(self) -> Optional[str]:
        """TODO"""
        return self._source_dir

    def resource_dir(self) -> Optional[str]:
        """TODO"""
        return self._resource_dir

    def log_dir(self) -> Optional[str]:
        """TODO"""
        return self._log_dir

    def get(self, property_name: str) -> Optional[str]:
        """TODO"""
        env_value = os.environ.get(Properties.environ_name(property_name))
        return str(env_value) if env_value else self._app_properties.get(property_name)

    def get_int(self, property_name: str) -> Optional[int]:
        """TODO"""
        env = os.environ.get(Properties.environ_name(property_name))
        return int(env) if env else self._app_properties.get_int(property_name)

    def get_float(self, property_name: str) -> Optional[float]:
        """TODO"""
        env = os.environ.get(Properties.environ_name(property_name))
        return float(env) if env else self._app_properties.get_float(property_name)

    def get_bool(self, property_name: str) -> Optional[bool]:
        """TODO"""
        env = os.environ.get(Properties.environ_name(property_name))
        return bool(env) if env else self._app_properties.get_bool(property_name)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.fetch
      @file: http_response.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from requests.models import CaseInsensitiveDict, Response

from hspylib.core.enums.charset import Charset
from hspylib.core.enums.content_type import ContentType
from hspylib.core.enums.http_code import HttpCode
from hspylib.core.enums.http_method import HttpMethod


class HttpResponse:
    """TODO"""

    @staticmethod
    def of(response: Response) -> 'HttpResponse':
        """TODO"""
        return HttpResponse(
            HttpMethod[response.request.method],
            response.url,
            HttpCode(response.status_code),
            response.text,
            response.headers,
            Charset(str(response.encoding).upper()) if response.encoding else None,
        )

    def __init__(self,
                 method: HttpMethod,
                 url: str,
                 status_code: HttpCode = None,
                 body: str = None,
                 headers: CaseInsensitiveDict = None,
                 encoding: Charset = Charset.UTF_8,
                 content_type=ContentType.APPLICATION_JSON):
        self.method = method
        self.url = url
        self.status_code = status_code
        self.body = body
        self.headers = headers
        self.encoding = encoding
        self.content_type = content_type
        if self.content_type:
            self.content_type.charset = self.encoding

    def __str__(self):
        return f"({self.status_code.value}) {self.status_code} {self.url}"

    def __repr__(self):
        return str(self.status_code.value)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   hspylib.main.hspylib
      @file: __main__.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import sys

from hspylib.addons.appman.app_extension import AppExtension
from hspylib.addons.appman.app_type import AppType
from hspylib.addons.appman.appman import AppManager
from hspylib.addons.widman.widman import WidgetManager
from hspylib.core.tools.commons import get_path, read_version, run_dir, syserr, sysout, dirname
from hspylib.modules.cli.application.application import Application
from hspylib.modules.cli.application.argument_chain import ArgumentChain

HERE = get_path(__file__)


class Main(Application):
    """HSPyLib Manager v{} - Manage HSPyLib applications."""

    # The hspylib version
    VERSION = read_version(f"{HERE}/.version")

    # HSPyLib manager usage message
    USAGE = (HERE / "usage.txt").read_text().format('.'.join(map(str, VERSION)))

    # The welcome message
    WELCOME = (HERE / "welcome.txt").read_text()

    def __init__(self, app_name: str):
        super().__init__(app_name, self.VERSION, self.USAGE, dirname(__file__))

    def _setup_parameters(self, *params, **kwargs) -> None:
        """Initialize application parameters and options"""
        if len(*params) == 0:
            welcome = self.WELCOME
            sysout(f"{welcome}")
            self.usage()
        else:
            # @formatter:off
            self._with_option('d', 'dest-dir', True)
            self._with_arguments(
                ArgumentChain.builder()
                    .when('operation', 'appman')
                        .require('app-name', '.+')
                        .require('app-type', '|'.join(AppType.values()))
                        .accept('app_ext', '.+')
                        .end()
                    .when('operation', 'widgets')
                        .accept('widget-name', '.+')
                        .accept('widget-args', '.+')
                        .end()
                    .build()
            )
            # @formatter:on

    def _main(self, *params, **kwargs) -> None:
        """Main entry point handler"""
        self._exec_application()

    def _exec_application(self) -> None:
        """Execute the application"""
        op = self.getarg('operation')
        if op == 'appman':
            manager = AppManager(self)
            app_type = AppType.of_value(self.getarg('app-type'))
            app_ext = self.getarg('app_ext').split(',') if self.getarg('app_ext') else []
            manager.create(
                self.getarg('app-name'),
                app_type,
                list(map(AppExtension.value_of, app_ext)),
                self.getopt('dest-dir') or run_dir())
        elif op == 'widgets':
            manager = WidgetManager(self)
            widget_name = self.getarg('widget-name')
            if widget_name:
                widget_args = str(self.getarg('widget-args')).split(',')
                manager.execute(widget_name, widget_args)
            else:
                manager.dashboard()
        else:
            syserr('### Invalid operation: {}'.format(op))
            self.usage(1)


# Application entry point
if __name__ == "__main__":
    Main('HSPyLib Manager').INSTANCE.run(sys.argv[1:])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.vt100
      @file: vt_colors.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from hspylib.core.enums.enumeration import Enumeration
from hspylib.modules.cli.vt100.vt_100 import Vt100


class VtColors(Enumeration):
    """TODO"""

    NC = Vt100.mode('0;0;0')
    BLACK = Vt100.mode('0;30')
    RED = Vt100.mode('0;31')
    GREEN = Vt100.mode('0;32')
    YELLOW = Vt100.mode('0;93')
    BLUE = Vt100.mode('0;34')
    PURPLE = Vt100.mode('35')
    CYAN = Vt100.mode('36')
    GRAY = Vt100.mode('38;5;8')
    ORANGE = Vt100.mode('38;5;202')
    VIOLET = Vt100.mode('0;95')
    WHITE = Vt100.mode('0;97')

    @staticmethod
    def colorize(input_string: str) -> str:
        return input_string \
            .replace("%NC%", VtColors.NC.code()) \
            .replace("%BLACK%", VtColors.BLACK.code()) \
            .replace("%RED%", VtColors.RED.code()) \
            .replace("%GREEN%", VtColors.GREEN.code()) \
            .replace("%YELLOW%", VtColors.YELLOW.code()) \
            .replace("%BLUE%", VtColors.BLUE.code()) \
            .replace("%PURPLE%", VtColors.PURPLE.code()) \
            .replace("%CYAN%", VtColors.CYAN.code()) \
            .replace("%GRAY%", VtColors.GRAY.code()) \
            .replace("%ORANGE%", VtColors.ORANGE.code()) \
            .replace("%VIOLET%", VtColors.VIOLET.code()) \
            .replace("%WHITE%", VtColors.WHITE.code())

    def code(self) -> str:
        return str(self)

    def placeholder(self) -> str:
        return f"%{self.name}%"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   hspylib.main.hspylib.addons.appman.templates
      @file: tpl-test_main.py
   @created: Tue, 1 Jun 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import logging as log
import os
import sys
import unittest

from hspylib.core.config.app_config import AppConfigs
from hspylib.core.tools.commons import dirname

TEST_DIR = dirname(__file__)


class TestMain(unittest.TestCase):

    # Setup tests
    def setUp(self):
        resource_dir = '{}/resources'.format(TEST_DIR)
        os.environ['ACTIVE_PROFILE'] = "test"
        self.configs = AppConfigs(
            source_root=TEST_DIR, resource_dir=resource_dir, log_dir=resource_dir
        )
        self.assertIsNotNone(self.configs)
        self.assertIsNotNone(AppConfigs.INSTANCE)
        log.info(self.configs)

    # Teardown tests
    def tearDown(self):
        pass

    # TEST CASES ----------

    def test_should_test_something(self):
        pass


# Program entry point.
if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestMain)
    unittest \
        .TextTestRunner(verbosity=2, failfast=True, stream=sys.stdout) \
        .run(suite)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.application
      @file: argument.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import re
from typing import Any


class Argument:

    @staticmethod
    def validate_chained_args(arg: Any) -> bool:
        """Validate all arguments following the chain path"""
        missing = 0
        next_arg = arg
        while next_arg:
            missing += 1 if not next_arg.value and next_arg.required else 0
            next_arg = next_arg.next_in_chain
        return missing == 0

    def __init__(
        self,
        name: str,
        validation_regex: str = '.*',
        required: bool = True,
        next_in_chain: Any = None):
        self.name = name
        self.validation_regex = validation_regex
        self.required = required
        self.next_in_chain = next_in_chain
        self.value = ''

    def __str__(self):
        return "arg_name: {}, val_regex: {}, required: {}, value: {}, next_in_chain => [{}]" \
            .format(self.name, self.validation_regex, self.required, self.value, self.next_in_chain)

    def __repr__(self):
        return str(self)

    def set_value(self, provided_arg: str) -> bool:
        """TODO"""
        self.value = provided_arg if re.match(rf'^({self.validation_regex})$', provided_arg) else None
        return bool(self.value)

    def set_next(self, argument: Any) -> None:
        """TODO"""
        self.next_in_chain = argument



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.icons.font_awesome
      @file: widget_icons.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from hspylib.modules.cli.icons.font_awesome.awesome import Awesome


class WidgetIcons(Awesome):
    """
        Dashboard UI icons.
        Codes can be found here:
        - https://fontawesome.com/cheatsheet?from=io
    """

    # @formatter:off
    WIDGET = u'\uFC65'    # ﱥ
    DATABASE = u'\uFB19'  # ﬙
    CLOCK = u'\uF651'     # 
    NETWORK = u'\uF819'   # 
    CHART_1 = u'\uFC67'   # ﱨ
    CHART_2 = u'\uFC68'   # ﱩ
    MUSIC = u'\uFC6E'     # ﱯ
    SIGN = u'\uFC7E'      # ﱿ
    SWORDS = u'\uFC84'    # ﲅ
    # @formatter:on



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.qt.promotions
      @file: hcombobox.py
   @created: Wed, 30 Jun 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from PyQt5.QtWidgets import QComboBox


class HComboBox(QComboBox):

    def set_item(self, item: str) -> None:
        """TODO"""
        if self.findText(item) < 0:
            self.addItem(item)

    def del_item(self, index: int = -1) -> None:
        """TODO"""
        if index < 0 or index >= self.count():
            if self.currentIndex() >= 0:
                super().removeItem(self.currentIndex())
        else:
            super().removeItem(index)



# _*_ coding: utf-8 _*_
#
# HSPyLib v0.11.1
#
# Package: main.hspylib.modules.cli.tui.extra.mdashboard
"""Package initialization."""

__all__ = [
  'dashboard_builder', 
  'dashboard_item', 
  'mdashboard'
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @Package: hspylib.main.hspylib.core.crud.db.mysql
      @file: mysql_repository.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import logging as log
import uuid
from abc import abstractmethod
from typing import List, Optional, Tuple

import pymysql
from pymysql.err import OperationalError, ProgrammingError
from requests.structures import CaseInsensitiveDict as SqlFilter

from hspylib.core.crud.crud_entity import CrudEntity
from hspylib.core.crud.db.db_repository import DBRepository
from hspylib.core.crud.db.sql_factory import SqlFactory
from hspylib.core.exception.exceptions import NotConnectedError


class MySqlRepository(DBRepository):
    """TODO"""

    _connection_cache = {}

    def __init__(self):
        super().__init__()
        self._connector = None
        self._cursor = None
        self._sql_factory = SqlFactory()

    def is_connected(self) -> bool:
        """TODO"""
        return self._connector is not None

    def connect(self) -> None:
        """TODO"""
        if not self.is_connected():
            cache_key = self.__str__()
            if cache_key in self._connection_cache:
                self._connector = self._connection_cache[cache_key]
                self._cursor = self._connector.cursor()
                if not self.is_connected():
                    raise ConnectionError("Not connected to the database")
            else:
                db_url = f"{self.user}@{self.hostname}:{self.port}/{self.database}"
                try:
                    self._connector = pymysql.connect(
                        host=self.hostname,
                        user=self.user,
                        port=self.port,
                        password=self.password,
                        database=self.database)
                    if not self.is_connected():
                        raise ConnectionError(f"Unable to connect to {db_url}")
                    self._cursor = self._connector.cursor()
                    log.debug("Connection to %s established", db_url)
                    self._connection_cache[cache_key] = self._connector
                except (OperationalError, ConnectionRefusedError) as err:
                    raise ConnectionError(f"Unable to connect to {db_url}") from err

    def disconnect(self) -> None:
        """TODO"""
        if self.is_connected():
            cache_key = self.__str__()
            self._connector.close()
            del self._connector
            del self._connection_cache[cache_key]
            log.debug('Disconnected from %s', str(self))
        else:
            raise NotConnectedError('Not connected to database.')

    def insert(self, entity: CrudEntity) -> None:
        """TODO"""
        if self.is_connected():
            entity.uuid = entity.uuid if entity.uuid is not None else str(uuid.uuid4())
            stm = self._sql_factory \
                .insert(entity) \
                .replace(':tableName', self.table_name())
            log.debug("Executing SQL statement: %s", stm)
            self.execute(stm, True)
        else:
            raise NotConnectedError('Not connected to database.')

    def update(self, entity: CrudEntity) -> None:
        """TODO"""
        if self.is_connected():
            stm = self._sql_factory \
                .update(entity, filters=SqlFilter({"UUID": '{}'.format(entity.uuid)})) \
                .replace(':tableName', self.table_name())
            log.debug('Executing SQL statement: %s', stm)
            self.execute(stm, True)
        else:
            raise NotConnectedError('Not connected to database.')

    def delete(self, entity: CrudEntity) -> None:
        """TODO"""
        if self.is_connected():
            stm = self._sql_factory \
                .delete(filters=SqlFilter({"UUID": '{}'.format(entity.uuid)})) \
                .replace(':tableName', self.table_name())
            log.debug('Executing SQL statement: %s', stm)
            self.execute(stm, True)
        else:
            raise NotConnectedError('Not connected to database.')

    def find_all(  # pylint: disable=arguments-differ
        self,
        column_set: List[str] = None,
        sql_filters: SqlFilter = None) -> Optional[list]:
        """TODO"""

        if self.is_connected():
            stm = self._sql_factory \
                .select(column_set=column_set, filters=sql_filters) \
                .replace(':tableName', self.table_name())
            log.debug('Executing SQL statement: %s', stm)
            self.execute(stm, True)
            result = self._cursor.fetchall()
            return list(map(self.row_to_entity, result)) if result else None

        raise NotConnectedError('Not connected to database.')

    def find_by_id(  # pylint: disable=arguments-differ
        self,
        column_set: List[str] = None,
        entity_id: str = None) -> Optional[CrudEntity]:
        """TODO"""

        if self.is_connected():
            if entity_id:
                stm = self._sql_factory \
                    .select(column_set=column_set, filters=SqlFilter({"UUID": '{}'.format(entity_id)})) \
                    .replace(':tableName', self.table_name())
                log.debug('Executing SQL statement: %s', stm)
                self.execute(stm, True)
                result = self._cursor.fetchall()
                if len(result) > 1:
                    raise ProgrammingError(f'Multiple results found {len(result)}')
                return self.row_to_entity(result[0]) if len(result) > 0 else None
            return None

        raise NotConnectedError('Not connected to database.')

    def execute(self, sql_statement: str, auto_commit: bool, *params) -> None:
        """TODO"""
        if self.is_connected():
            log.debug("Executing SQL statement: %s with params: [%s]", sql_statement, ', '.join(params))
            self._cursor.execute(sql_statement, *params)
            if auto_commit:
                self.commit()
        else:
            raise NotConnectedError('Not connected to database.')

    def commit(self) -> None:
        """TODO"""
        log.debug('Committing database changes')
        self._connector.commit()

    def rollback(self) -> None:
        """TODO"""
        log.debug('Rolling back database changes')
        self._connector.rollback()

    @abstractmethod
    def row_to_entity(self, row: Tuple) -> CrudEntity:
        pass

    @abstractmethod
    def table_name(self) -> str:
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.core.tools
   @file: git_tools.py
  @created: Nov 14, 2019
   @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior
     @site: https://github.com/yorevs/homesetup
  @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from hspylib.modules.cli.vt100.terminal import Terminal


class GitTools:
    """TODO"""

    @staticmethod
    def top_level_dir() -> str:
        """TODO"""
        return Terminal.shell_exec('git rev-parse --show-toplevel')

    @staticmethod
    def current_branch() -> str:
        """TODO"""
        return Terminal.shell_exec('git symbolic-ref --short HEAD')

    @staticmethod
    def changelog(from_tag: str, to_tag: str) -> str:
        """TODO"""
        return Terminal.shell_exec(f"git log --oneline --pretty='format:%h %ad %s' --date=short {from_tag}^..{to_tag}^")

    @staticmethod
    def unreleased() -> str:
        """TODO"""
        latest_tag = Terminal.shell_exec('git describe --tags --abbrev=0 HEAD^')
        return Terminal.shell_exec(f"git log --oneline --pretty='format:%h %ad %s' --date=short '{latest_tag}'..HEAD")

    @staticmethod
    def release_date(tag_name: str) -> str:
        """TODO"""
        return Terminal.shell_exec(f"git log -1 --pretty='format:%ad' --date=short {tag_name}")

    @staticmethod
    def tag_list() -> str:
        """TODO"""
        return Terminal.shell_exec("git log --tags --simplify-by-decoration --pretty='format:%ci %d'")

    @staticmethod
    def create_tag(version: str, commit_id: str = 'HEAD', description: str = None) -> str:
        """TODO"""
        return Terminal.shell_exec(f"git tag -a v{version} {commit_id} -m '{description or version}'")

    @staticmethod
    def search_logs(filter_by: str = '.*') -> str:
        """TODO"""
        return Terminal.shell_exec(f"git log --grep='{filter_by}' --pretty=format:'%h %ad %s' --date=short")

    @staticmethod
    def show_file(filename: str, commit_id: str = 'HEAD') -> str:
        return Terminal.shell_exec(f"git show {commit_id}:{filename}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.qt.views
      @file: main_qt_view.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from abc import abstractmethod

from PyQt5 import uic

from hspylib.modules.qt.views.qt_view import QtView


class MainView(QtView):
    """TODO"""

    def __init__(self, ui_file_path: str):
        form, window = uic.loadUiType(ui_file_path)
        super().__init__(window)
        self.form = form
        self.form.setupUi(window)
        self._setup_ui()

    @abstractmethod
    def _setup_ui(self) -> None:
        """TODO"""

    def show(self) -> None:
        """TODO"""
        self.window.show()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.vt100
      @file: vt_codes.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import re
from enum import auto

from hspylib.core.enums.enumeration import Enumeration
from hspylib.modules.cli.vt100.vt_100 import Vt100


def vt_print(vt100_str: str, end: str = '') -> None:
    """Print a vt-100 encoded string. VT-100 string will contain one or more %VT-100-CODE% """
    print(VtCodes.decode(vt100_str), end=end)


class VtCodes(Enumeration):
    """VT-100 escape codes"""

    # @formatter:off
    CSV = Vt100.save_cursor()           # ^[7 -> Save cursor position and attributes
    CRE = Vt100.restore_cursor()        # ^[8 -> Restore cursor position and attributes
    RIS = Vt100.reset()                 # ^[c -> Reset terminal to initial state

    SAW = Vt100.set_auto_wrap(True)     # ^[?7h  -> Set auto-wrap mode
    UAW = Vt100.set_auto_wrap(False)    # ^[?7l  -> Unset auto-wrap mode
    SSC = Vt100.set_show_cursor(True)   # ^[?25h -> Set show cursor
    USC = Vt100.set_show_cursor(False)  # ^[?25l -> Unset show cursor

    ED0 = Vt100.clear_screen()          # ^[[J  -> Clear screen from cursor down
    ED1 = Vt100.clear_screen(1)         # ^[[1J -> Clear screen from cursor up
    ED2 = Vt100.clear_screen(2)         # ^[[2J -> Clear entire screen

    EL0 = Vt100.clear_line()            # ^[[K  -> Clear line from cursor right
    EL1 = Vt100.clear_line(1)           # ^[[1K -> Clear line from cursor left
    EL2 = Vt100.clear_line(2)           # ^[[2K -> Clear entire line

    HOM = Vt100.cursor_pos()            # ^[[H  -> Move cursor to upper left corner

    # The following entries must defined as auto(), so they can be invoked as Callable

    MOD = auto()                        # ^[[<m1;m2;m3>m  -> Set terminal modes
    CUP = auto()                        # ^[[<v>;<h>H     -> Move cursor to screen location <v,h>
    CUU = auto()                        # ^[[<n>A         -> Move cursor up n lines
    CUD = auto()                        # ^[[<n>B         -> Move cursor down n lines
    CUF = auto()                        # ^[[<n>C         -> Move cursor right n lines
    CUB = auto()                        # ^[[<n>D         -> Move cursor left n lines
    # @formatter:on

    # For all mnemonics that take arguments we need to include in this map, so we can call it
    __VT100_FNC_MAP__ = {
        "MOD": Vt100.mode,
        "CUP": Vt100.cursor_pos,
        "CUU": Vt100.cursor_move_up,
        "CUD": Vt100.cursor_move_down,
        "CUF": Vt100.cursor_move_forward,
        "CUB": Vt100.cursor_move_backward,
    }

    @classmethod
    def decode(cls, input_string: str) -> str:
        """Decode the string into a VT_CODE enum"""
        results = re.findall(r'%([a-zA-Z0-9]+)(\([0-9]+(;[0-9]+)*\))?%', input_string)
        for nextResult in results:
            mnemonic = nextResult[0]
            if mnemonic in VtCodes.names():
                args = nextResult[1][1:-1] if nextResult[1] else ''
                if args:
                    input_string = input_string.replace(
                        '%{}%'.format(mnemonic + nextResult[1]), VtCodes.value_of(mnemonic)(args))
                else:
                    input_string = input_string.replace(
                        '%{}%'.format(mnemonic), VtCodes.value_of(mnemonic).value)

        return input_string

    def __call__(self, *args, **kwargs) -> str:
        return VtCodes.__VT100_FNC_MAP__[self.name](args[0])

    def __str__(self) -> str:
        return str(self.value)

    def code(self) -> str:
        """Decode the string into a VT_CODE enum"""
        return str(self)

    def placeholder(self) -> str:
        """Decode the string into a VT_CODE enum"""
        return f"%{self.name}%"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.qt
      @file: qt_application.py
   @created: Wed, 30 Jun 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from abc import ABC
from typing import TypeVar

from PyQt5.QtWidgets import QApplication

V = TypeVar('V')


class QtApplication(ABC):

    def __init__(self, main_view: V):
        super().__init__()
        self.app = QApplication([])
        self.main_view = main_view()

    def run(self) -> int:
        """Show the main Qt application Widget"""
        self.main_view.show()
        return self.app.exec_()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.core.tools
      @file: commons.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""
import inspect
import logging as log
import os
import pathlib
import re
import sys
from datetime import timedelta, datetime
from typing import Any, List, Optional, Tuple, Type, Union

from hspylib.core.tools.constants import DATE_TIME_FORMAT
from hspylib.core.tools.validator import Validator
from hspylib.modules.cli.vt100.vt_codes import VtCodes
from hspylib.modules.cli.vt100.vt_colors import VtColors

LOG_FMT = '{} {} {} {}{} {} '.format(
    '%(asctime)s',
    '[%(threadName)-10.10s]',
    '%(levelname)-5.5s',
    '%(filename)s::',
    '%(funcName)s(@Line:%(lineno)d)',
    '%(message)s'
)


def _reset_logger():
    """TODO"""
    # Remove handlers if there is any.
    root = log.getLogger()
    if root.handlers:
        for handler in root.handlers:
            root.removeHandler(handler)


def log_init(
    log_file: str,
    create_new: bool = True,
    mode: str = 'a',
    level: int = log.DEBUG,
    filename: str = LOG_FMT) -> bool:
    """Initialize the system logger
    :param log_file: TODO
    :param create_new:  TODO
    :param mode:  TODO
    :param level:  TODO
    :param filename:  TODO
    """
    # if someone tried to log something before log_init is called, Python creates a default handler that
    _reset_logger()

    create = bool(create_new or not os.path.exists(log_file))
    with open(log_file, 'w' if create else 'a'):
        os.utime(log_file, None)
        log.basicConfig(
            filename=log_file,
            format=filename,
            level=level,
            filemode=mode)
        log.info(
            "Logger init. filename=%s level=%s mode=%s append=%s", os.path.basename(log_file), level, mode, not create)

    return os.path.exists(log_file)


def is_debugging():
    for frame in inspect.stack():
        if frame[1].endswith("pydevd.py"):
            return True
    return False


def now() -> str:
    """TODO"""
    return datetime.now().strftime(DATE_TIME_FORMAT)


def now_ms() -> int:
    return int(datetime.now().timestamp())


def read_version(version_filepath: str = ".version") -> Tuple:
    """Retrieve the version from the version file in the form: Tuple[major,minor,build]"""
    try:
        log.info("Reading version from %s", version_filepath)
        with open(version_filepath) as fh_version:
            ver = tuple(map(str.strip, fh_version.read().split('.')))
            return ver if ver else (0, 0, 0)
    except FileNotFoundError:
        return 0, 0, 0


def dirname(filepath: str) -> str:
    """Retrieve the directory of the specified filepath"""
    return os.path.dirname(os.path.realpath(filepath))


def run_dir() -> str:
    """Retrieve the application's root directory"""
    return sys.path[0]


def get_path(filepath: str) -> pathlib.Path:
    """Retrieve the Path of the file"""
    return pathlib.Path(filepath).parent


def sysout(string: str, end: str = '\n') -> None:
    """Print the unicode input_string decoding vt100 placeholders
    :param string: values to be printed to sys.stdout
    :param end: string appended after the last value, default a newline
    """
    if Validator.is_not_blank(string):
        msg = VtColors.colorize(VtCodes.decode(f"{string}%NC%"))
        print(msg, file=sys.stdout, flush=True, end=end)


def syserr(string: str, end: str = '\n') -> None:
    """Print the unicode input_string decoding vt100 placeholders
    :param string: values to be printed to sys.stderr
    :param end: string appended after the last value, default a newline
    """
    if Validator.is_not_blank(string):
        msg = VtColors.colorize(VtCodes.decode(f"%RED%{string}%NC%"))
        print(msg, file=sys.stderr, flush=True, end=end)


def class_attribute_names(clazz: Type) -> Optional[Tuple]:
    """TODO
    :param clazz: TODO
    """
    instance = clazz()
    return tuple(vars(instance).keys()) if clazz else None


def class_attribute_values(instance: dict) -> Optional[Tuple]:
    """TODO
    :param instance: TODO
    """
    return tuple(instance.values()) if instance else None


def new_dynamic_object(type_name: str, inherited_type: Any = object):
    """TODO
    :param: type_name: TODO
    :param: inherited_type: TODO
    """

    return type(type_name, (inherited_type,), {})()


def split_and_filter(input_str: str, regex_filter: str = '.*', delimiter: str = '\n') -> List[str]:
    """Split the string using the delimiter and filter using the specified regex filter
    :param input_str: The string to be split
    :param regex_filter: The regex to filter the string
    :param delimiter: The delimiter according which to split the string
    """
    regex = re.compile(regex_filter)
    result_list = list(filter(regex.search, input_str.split(delimiter)))

    return result_list


def flatten_dict(dictionary: dict, parent_key='', sep='.') -> dict:
    """TODO
    :param dictionary:
    :param parent_key:
    :param sep:
    :return:
    """
    flat_dict = {}
    for key, value in dictionary.items():
        new_key = parent_key + sep + key if parent_key else key
        if isinstance(value, dict):
            flat_dict.update(flatten_dict(value, new_key, sep=sep).items())
        else:
            flat_dict.update({new_key: value})

    return flat_dict


def search_dict(root_element: dict, search_path: str, parent_key='', sep='.') -> Optional[Any]:
    """
    TODO
    :param root_element:
    :param search_path:
    :param parent_key:
    :param sep:
    :return:
    """
    found = search_path == parent_key
    el = None
    if not found and isinstance(root_element, dict):
        for key, value in root_element.items():
            if found:
                break
            el_path = parent_key + sep + key if parent_key else key
            if search_path == el_path:
                found, el = True, value
                break
            if isinstance(value, dict):
                found, el = search_dict(value, search_path, el_path, sep=sep)
            elif isinstance(value, list):
                for next_val in value:
                    found, el = search_dict(next_val, search_path, el_path, sep=sep)
            # Skip if the element is a leaf

    return found, el

def get_or_default(options: Union[tuple, list], index: int, default_value=None) -> Optional[Any]:
    """Retrieve an item from the options list or default_value if index is out of range
    :param options: The available list of options
    :param index: The index of the item
    :param default_value: The default value if the index is out of range
    """
    return options[index] if index < len(options) else default_value


def get_by_key_or_default(options: dict, key: str, default_value=None) -> Optional[Any]:
    """Retrieve an item from the options list or default_value if key was not found
    :param options: The available list of options
    :param key: The key of the item
    :param default_value: The default value if the index is not found
    """
    return options[key] if key in options else default_value


def str_to_bool(string: str, true_values: List[str] = None) -> bool:
    """Convert a string to boolean
    :param string: The string to be converted
    :param true_values: The list of strings that will become True value
    """
    if true_values is None:
        true_values = [
            'true', 'on', 'yes', '1', 'y'
        ]
    return string.lower() in true_values if string else False


def safe_del_file(filename: str, on_not_found_except: bool = False) -> bool:
    """TODO
    :param filename:
    :param on_not_found_except:
    """
    if os.path.exists(filename):
        os.remove(filename)
        return True

    if on_not_found_except:
        raise FileNotFoundError('File was not found on the system: {}'.format(filename))

    return False


def file_is_not_empty(filename: str) -> bool:
    """TODO
    :param filename:
    """
    return os.path.exists(filename) and os.stat(filename).st_size > 0


def touch_file(filename: str) -> None:
    """TODO
    :param filename:
    """
    with open(filename, 'a'):
        os.utime(filename, None)


def human_readable_bytes(size_in_bytes: int) -> Tuple[str, str]:
    """TODO
    :param size_in_bytes:
    """

    byte_size = float(size_in_bytes)
    kb, mb, gb, tb = 2 ** 10, 2 ** 20, 2 ** 30, 2 ** 40

    if 0 <= byte_size <= kb:
        ret_val = '%3.2f' % byte_size
        ret_unit = '[B]'
    elif kb < byte_size <= mb:
        ret_val = '%3.2f' % (byte_size / kb)
        ret_unit = '[Kb]'
    elif mb < byte_size <= gb:
        ret_val = '%3.2f' % (byte_size / mb)
        ret_unit = '[Mb]'
    elif gb < byte_size <= tb:
        ret_val = '%3.2f' % (byte_size / gb)
        ret_unit = '[Gb]'
    else:
        ret_val = '%3.2f' % (byte_size / tb)
        ret_unit = '[Tb]'

    return ret_val, ret_unit


def human_readable_time(time_microseconds: int) -> str:
    """TODO
    :param time_microseconds:
    """
    delta = timedelta(microseconds=time_microseconds)
    total_seconds = delta.seconds
    seconds = total_seconds % 60
    minutes = total_seconds / 60 % 60
    hours = total_seconds / 3600
    microseconds = delta.microseconds
    # Using format: HH:MM:SS.uuuuuu
    str_line = f"{hours:02d}:{minutes:02d}:{seconds:02d}.{microseconds:06d}"

    return str_line


def build_url(url_or_part: str) -> str:
    """TODO
    :param url_or_part:
    """
    if re.match(r'https?://.*', url_or_part):
        return url_or_part

    return f"http://{url_or_part}"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.icons.font_awesome
      @file: dashboard_icons.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from hspylib.modules.cli.icons.font_awesome.awesome import Awesome


class DashboardIcons(Awesome):
    """
        Dashboard UI icons.
        Codes can be found here:
        - https://fontawesome.com/cheatsheet?from=io
    """

    # @formatter:off
    SEARCH = u'\uF002'       # 
    EMAIL = u'\uF003'        # 
    LIKE = u'\uF004'         # 
    USER = u'\uF007'         # 
    MOVIE = u'\uF008'        # 
    DASHBOARD = u'\uF00A'    # 
    POWER = u'\uF011'        # 
    SETTINGS = u'\uF013'     # 
    ADJUSTMENTS = u'\uFB2D'  # שּׂ
    HOME = u'\uF015'         # 
    DOCUMENT = u'\uF016'     # 
    HISTORY = u'\uF017'      # 
    DELIVERED = u'\uFB3D'    # ﬽
    READ = u'\uFB3C'         # לּ
    DOWNLOAD = u'\uF019'     # 
    ARCHIVE = u'\uF01C'      # 
    PLAY = u'\uF01D'         # 
    FLAG = u'\uF024'         # 
    FAVORITE = u'\uF02E'     # 
    PRINTER = u'\uF02F'      # 
    CAMERA = u'\uF030'       # 
    LIST = u'\uF03A'         # 
    CODE = u'\uFABF'         # 謹
    FORM = u'\uF767'         # 
    FORMAT = u'\uF9C5'       # 暈
    PICTURE = u'\uF03E'      # 
    ERROR = u'\uF06A'        # 
    ALERT = u'\uF071'        # 
    AGENDA = u'\uF073'       # 
    CHAT = u'\uF075'         # 
    SHOPPING = u'\uF07A'     # 
    GRAPHS = u'\uF080'       # 
    FOLDER = u'\uF74A'       # 
    FOLDER_OPEN = u'\uF07C'  # 
    EXIT = u'\uF08B'         # 
    SEND = u'\uF989'         # 黎
    SENT = u'\uF1D8'         # 
    COMPASS = u'\uF124'      # 
    TWITTER = u'\uF081'      # 
    FACEBOOK = u'\uF082'     # 
    LINKED_IN = u'\uF08C'    # 
    GRAPH = u'\uF08F'        # 
    SAVE = u'\uF0C7'         # 
    CLOUD_DOWN = u'\uF0ED'   # 
    CLOUD_UP = u'\uF0EE'     # 
    NOTIFICATION = u'\uF0F3' # 
    DATABASE = u'\uF1C0'     # 
    PLUGIN = u'\uF930'       # 擄
    PLUG_IN = u'\uF1E6'      # 
    PLUG_OUT = u'\uF492'     # 
    UNPLUGGED = u'\uFBA4'    # ﮤ
    PLUGGED = u'\uFBA3'      # ﮣ
    KILL = u'\uFB8A'         # ﮊ
    IDENTITY = u'\uF2BB'     # 
    DOCKER = u'\uF308'       # 
    MUSIC = u'\uF3B5'        # 
    MUTE = u'\uF86B'         # 
    UNMUTE = u'\uF86C'       # 
    SECRET = u'\uF49C'       # 
    IE = u'\uF7FF'           # 
    GIT = u'\uF813'          # 
    VERIFIED = u'\uFC8F'     # ﲏ
    LIGHT_ON = u'\uF834'     # 
    LIGHT_OFF = u'\uF835'    # 
    PYTHON = u'\uF81F'       # 
    JAVA = u'\uF4E4'         # 
    CPLUSPLUS = u'\uFB70'    # ﭱ
    GITHUB = u'\uF408'       # 
    CLOUD_PUT = u'\uF40A'    # 
    CLOUD_GET = u'\uF409'    # 
    EXPORT = u'\uF5F6'       # 
    IMPORT = u'\uF5F9'       # 

    # @formatter:on


if __name__ == '__main__':
    DashboardIcons.demo_icons()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.core.metaclass
      @file: singleton.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from typing import Type


class Singleton(type):
    """TODO"""

    _instances = {}

    def __call__(cls, *args, **kwargs):
        """TODO"""
        if not cls.has_instance(cls):
            instance = super(Singleton, cls).__call__(*args, **kwargs)
            cls.INSTANCE = instance
            cls._instances[cls] = instance

        return cls._instances[cls]

    @classmethod
    def instances(mcs) -> dict:  # pylint: disable=bad-mcs-classmethod-argument
        """TODO"""
        return Singleton._instances

    @classmethod
    def has_instance(mcs, cls) -> bool:  # pylint: disable=bad-mcs-classmethod-argument
        """TODO"""
        return cls in Singleton._instances

    @classmethod
    def del_instance(mcs, clazz: Type) -> bool:  # pylint: disable=bad-mcs-classmethod-argument
        """TODO"""
        singleton_instances = Singleton.__getattribute__(mcs, '_instances')
        if clazz in singleton_instances:
            del mcs._instances[clazz]
            delattr(clazz, 'INSTANCE')
            del clazz
            return True

        return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.vt100
      @file: vt_100.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import re
from abc import ABC

from hspylib.core.tools.preconditions import check_argument


class Vt100(ABC):
    """
    References:
        - https://vt100.net/docs/vt100-ug/chapter3.html
        - https://espterm.github.io/docs/VT100%20escape%20codes.html
    """

    # Esc<Sequence>
    @classmethod
    def escape(cls, seq: str) -> str:
        """TODO"""
        return f"\033{seq}"

    # Esc[<Code>
    @classmethod
    def sequence(cls, code: str) -> str:
        """TODO"""
        return cls.escape(f"[{code}")

    # Esc7
    @classmethod
    def save_cursor(cls) -> str:
        """TODO"""
        return cls.escape('7')

    # Esc8
    @classmethod
    def restore_cursor(cls) -> str:
        """TODO"""
        return cls.escape('8')

    # Esc[c
    @classmethod
    def reset(cls) -> str:
        """TODO"""
        return cls.sequence('c')

    # Esc[?7<h/l>
    @classmethod
    def set_auto_wrap(cls, enabled: bool) -> str:
        """TODO"""
        return cls.sequence(f"?7{'h' if enabled else 'l'}")

    # Esc[?25<h/l>
    @classmethod
    def set_show_cursor(cls, enabled: bool) -> str:
        """TODO"""
        return cls.sequence(f"?25{'h' if enabled else 'l'}")

    # Esc[<Modes...>m
    @classmethod
    def mode(cls, mod_seq: str) -> str:
        """TODO"""
        check_argument(bool(re.match(r"[0-9]+(;[0-9]+){0,2}", mod_seq)), 'Invalid mode sequence')
        return cls.sequence(f"{mod_seq}m")

    # Esc[<n>J
    @classmethod
    def clear_screen(cls, mod_cls: int = None) -> str:
        """TODO"""
        if mod_cls is None:
            return cls.sequence('J')

        check_argument(mod_cls in [0, 1, 2], 'Invalid mode sequence')
        return cls.sequence(f'{mod_cls}J')

    # Esc[<n>K
    @classmethod
    def clear_line(cls, mod_cls: int = None) -> str:
        """TODO"""
        if mod_cls is None:
            return cls.sequence('K')

        check_argument(mod_cls in [0, 1, 2], 'Invalid mode sequence')
        return cls.sequence(f'{mod_cls}K')

    # Esc[<v>;<h>H
    @classmethod
    def cursor_pos(cls, cup_seq: str = None) -> str:
        """TODO"""
        if cup_seq is None:
            return cls.sequence('H')

        check_argument(bool(re.match(r"[0-9]*;[0-9]*", cup_seq)), 'Invalid position sequence')
        return cls.sequence(f"{cup_seq}H")

    # Esc[<n><A/B/C/D>
    @classmethod
    def cursor_move(cls, amount: int, direction: str) -> str:
        """TODO"""
        check_argument(int(amount) >= 0 and direction in ['A', 'B', 'C', 'D'], 'Invalid direction or move amount')
        return cls.sequence(f"{amount}{direction}")

    # Esc[<n>A
    @classmethod
    def cursor_move_up(cls, amount: int = None) -> str:
        """TODO"""
        return cls.cursor_move(amount if amount else 0, 'A')

    # Esc[<n>B
    @classmethod
    def cursor_move_down(cls, amount: int = None) -> str:
        """TODO"""
        return cls.cursor_move(amount if amount else 0, 'B')

    # Esc[<n>C
    @classmethod
    def cursor_move_forward(cls, amount: int = None) -> str:
        """TODO"""
        return cls.cursor_move(amount if amount else 0, 'C')

    # Esc[<n>D
    @classmethod
    def cursor_move_backward(cls, amount: int = None) -> str:
        """TODO"""
        return cls.cursor_move(amount if amount else 0, 'D')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.mock
      @file: mock_server_handler.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import logging as log
from http.server import BaseHTTPRequestHandler
from typing import List, Tuple

from requests.structures import CaseInsensitiveDict

from hspylib.core.enums.content_type import ContentType
from hspylib.core.enums.http_code import HttpCode
from hspylib.core.enums.http_method import HttpMethod


class MockServerHandler(BaseHTTPRequestHandler):
    """TODO"""

    def __init__(self, request: bytes, client_address: Tuple[str, int], parent):
        self.parent = parent
        super().__init__(request, client_address, parent)

    @staticmethod
    def remove_reserved_headers(headers: CaseInsensitiveDict) -> dict:
        """TODO"""

        filtered = {}
        if headers:
            reserved = ['content-size', 'server', 'date']
            for header in headers:
                if header.lower() not in reserved:
                    filtered[header] = headers[header]
        return filtered

    def process_headers(
        self,
        headers: CaseInsensitiveDict = None,
        content_type: ContentType = ContentType.APPLICATION_JSON,
        content_length: int = 0) -> None:
        """TODO"""

        headers = MockServerHandler.remove_reserved_headers(headers)
        if headers and len(headers) > 0:
            for key, value in headers.items():
                self.send_header(key, value)
        self.send_header("Server", 'MockServer v{}'.format(self.parent.version))
        self.send_header("Date", self.date_time_string())
        self.send_header("Content-Type", str(content_type))
        self.send_header("Content-Length", str(content_length))
        self.end_headers()

    def process_default(
        self,
        code: HttpCode = HttpCode.OK,
        content_type: ContentType = ContentType.APPLICATION_JSON,
        headers: CaseInsensitiveDict = None) -> None:
        """TODO"""

        log.debug('Processing a default request status_code=%s content-type=%s', code, content_type)
        self.send_response_only(code.value)
        self.process_headers(headers, content_type)

    def process_request(self, method: HttpMethod) -> None:
        """TODO"""

        if self.parent.is_allowed(method):
            request = self.parent.mock(method, self.path)
            if request:
                log.debug('Processing a request status_code=%s content-type=%s',
                          request.status_code, request.content_type)
                if not request.status_code:
                    code = HttpCode.INTERNAL_SERVER_ERROR.value
                    request.body = 'Mocked response status_code must be provided'
                elif method in ['OPTIONS', 'HEAD']:
                    code = request.status_code.value
                    request.body = None
                else:
                    code = request.status_code.value
                headers = request.headers if request.headers else []
                self.send_response_only(code)
                if request.received_body and 'Content-Length' in self.headers:
                    length = int(self.headers['Content-Length'])
                    request.body = self.rfile.read(length).decode(str(request.encoding))
                    self.process_headers(headers, request.content_type, length)
                else:
                    self.process_headers(headers, request.content_type, len(request.body) if request.body else 0)
                if request.body:
                    self.wfile.write(request.body.encode(str(request.encoding)))
            else:
                self.process_default(HttpCode.NOT_FOUND)
        else:
            self.process_default(HttpCode.METHOD_NOT_ALLOWED)

    def find_allowed_methods(self) -> List[str]:
        """TODO"""

        allowed_methods = ['OPTIONS']
        if self.parent.is_allowed(HttpMethod.HEAD):
            allowed_methods.append('HEAD')
        if self.parent.is_allowed(HttpMethod.GET):
            allowed_methods.append('GET')
        if self.parent.is_allowed(HttpMethod.POST):
            allowed_methods.append('POST')
        if self.parent.is_allowed(HttpMethod.PUT):
            allowed_methods.append('PUT')
        if self.parent.is_allowed(HttpMethod.PATCH):
            allowed_methods.append('PATCH')
        if self.parent.is_allowed(HttpMethod.DELETE):
            allowed_methods.append('DELETE')
        return allowed_methods

    # Do not rename the methods below because the serves uses this pattern do handle requests

    def do_OPTIONS(self) -> None:
        """ Handles: OPTIONS requests. Due to the base class, this name will not follow the naming conventions"""
        mock_request = self.parent.mock(HttpMethod.OPTIONS, self.path)
        headers = CaseInsensitiveDict({'Allow': ', '.join(self.find_allowed_methods())})
        if mock_request:
            self.process_request(mock_request)
        else:
            self.process_default(HttpCode.NO_CONTENT, headers=headers)

    def do_HEAD(self):
        """ Handles: HEAD requests. Due to the base class, this name will not follow the naming conventions"""
        self.process_request(HttpMethod.HEAD)

    def do_GET(self):
        """ Handles: GET requests. Due to the base class, this name will not follow the naming conventions"""
        self.process_request(HttpMethod.GET)

    def do_POST(self):
        """ Handles: POST requests. Due to the base class, this name will not follow the naming conventions"""
        self.process_request(HttpMethod.POST)

    def do_PUT(self):
        """ Handles: PUT requests. Due to the base class, this name will not follow the naming conventions"""
        self.process_request(HttpMethod.PUT)

    def do_PATCH(self):
        """ Handles: PATCH requests. Due to the base class, this name will not follow the naming conventions"""
        self.process_request(HttpMethod.PATCH)

    def do_DELETE(self):
        """ Handles: DELETE requests. Due to the base class, this name will not follow the naming conventions"""
        self.process_request(HttpMethod.DELETE)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.application
      @file: application.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""
import atexit
import getopt
import logging as log
import signal
import sys
from datetime import datetime
from typing import Callable, List, Optional, Set, Tuple

from hspylib.core.config.app_config import AppConfigs
from hspylib.core.exception.exceptions import InvalidArgumentError, InvalidOptionError
from hspylib.core.metaclass.singleton import Singleton
from hspylib.core.tools.commons import sysout
from hspylib.modules.cli.application.argument import Argument
from hspylib.modules.cli.application.argument_chain import ArgumentChain
from hspylib.modules.cli.application.option import Option


class Application(metaclass=Singleton):
    """HSPyLib application framework"""

    VERSION = None
    USAGE = None

    @staticmethod
    def exit_handler(signum=0, frame=None, clear_screen: bool = False) -> None:
        """
        Handle interruptions to shutdown gracefully
        :param signum: The signal number or the exit code
        :param frame: The frame raised by the signal
        :param clear_screen: Whether to clean the screen before execution or not
        """
        if frame is not None:
            log.warning('Signal handler hooked signum=%d frame=%s', signum, frame)
            exit_code = 3
        else:
            log.info('Exit handler called')
            exit_code = signum
        if clear_screen:
            sysout('%ED2%%HOM%')
        sys.exit(exit_code)

    def __init__(
        self,
        app_name: str = None,
        app_version: Tuple[int, int, int] = None,
        app_usage: str = None,
        source_dir: str = None,
        resource_dir: str = None,
        log_dir: str = None):

        signal.signal(signal.SIGINT, self.exit_handler)
        signal.signal(signal.SIGTERM, self.exit_handler)

        self._app_name = app_name
        self._app_version = app_version
        self._app_usage = app_usage
        self._options = {}
        self._arguments = []
        self._args = {}
        self._opts = {}

        if source_dir:
            self.configs = AppConfigs(
                source_root=source_dir,
                resource_dir=resource_dir,
                log_dir=log_dir
            )
        if app_usage:
            self._with_option('h', 'help', handler=self.usage)
        if app_version:
            self._with_option('v', 'version', handler=self.version)

    def run(self, *params, **kwargs) -> None:
        """Main entry point handler"""
        log.info('Run started %s', datetime.now())
        try:
            atexit.register(self._cleanup)
            self._setup_parameters(*params, **kwargs)
            self._parse_parameters(*params, **kwargs)
            self._main(*params, **kwargs)
        except (InvalidOptionError, InvalidArgumentError) as err:
            self.usage(exit_code=1, no_exit=True)
            raise err  # Re-Raise the exception so upper level layers can catch
        finally:
            log.info('Run finished %s', datetime.now())

    def usage(self, exit_code: int = 0, no_exit: bool = False) -> None:
        """Display the usage message and exit with the specified code ( or zero as default )
        :param no_exit: Do no exit the application on usage call
        :param exit_code: The exit code
        """
        sysout(self._app_usage)
        if not no_exit:
            self.exit_handler(exit_code)

    def version(self, exit_code: int = 0, no_exit: bool = False) -> None:
        """Display the current program version and exit
        :param no_exit: Do no exit the application on usage call
        :param exit_code: The exit code
        """
        sysout('{} v{}'.format(self._app_name, '.'.join(map(str, self._app_version))))
        if not no_exit:
            self.exit_handler(exit_code)

    def getopt(self, opt_name: str) -> Optional[str]:
        """Get the option value named by the opt_name"""
        return next((val for opt, val in self._opts.items() if opt == opt_name), None)

    def getarg(self, arg_name: str) -> Optional[str]:
        """Get the argument value named by the opt_name"""
        return next((val for arg, val in self._args.items() if arg == arg_name), None)

    def _setup_parameters(self, *params, **kwargs) -> None:  # pylint: disable=unused-argument,no-self-use
        """Initialize application parameters and options"""
        log.info('setup_parameters was not overridden')

    def _main(self, *params, **kwargs) -> None:  # pylint: disable=unused-argument,no-self-use
        """Execute the application's main statements"""
        log.info('main was not overridden')

    def _cleanup(self) -> None:  # pylint: disable=no-self-use
        """Execute code cleanup before exiting"""
        log.info('cleanup was not overridden')

    def _with_option(
        self,
        shortopt: chr,
        longopt: str,
        has_argument: bool = False,
        handler: Callable = None) -> None:
        """Specify an option for the command line"""
        self._options[longopt] = Option(shortopt, longopt, has_argument, handler)

    def _with_arguments(
        self,
        chained_args: Set[ArgumentChain.ChainedArgument]) -> None:
        """Specify an argument for the command line"""
        self._arguments = chained_args

    def _parse_parameters(self, parameters: List[str]) -> None:
        """ Handle program parameters.
        :param parameters: The list of unparsed program parameters passed by the command line
        """
        # First parse all options, then, arguments
        self._parse_arguments(self._parse_options(parameters))

    def _parse_options(self, parameters: List[str]) -> List[str]:
        """Handle program options in the form: Short opts: -<C>, Long opts: --<Word>
        :param parameters: The list of unparsed program parameters passed by the command line
        :return A list of remaining arguments
        """
        try:
            opts, args = getopt.getopt(parameters, self._shortopts(), self._longopts())
            for op, arg in opts:
                option = self._find_option(op)
                if option:
                    if option.cb_handler:
                        option.cb_handler(arg)
                    self._opts[option.name] = arg
                else:
                    raise InvalidOptionError(f'Option "{op}" not recognized')
            return args
        except getopt.GetoptError as err:
            raise InvalidOptionError(f"### Invalid option: {str(err)}") from err

    def _find_option(self, item: str) -> Optional[Option]:
        """Getter for application option"""
        return next((op for key, op in self._options.items() if op == item), None)

    def _parse_arguments(self, provided_args: List[str]) -> None:
        """Handle program arguments
        :param provided_args: Arguments left after processing all options
        """
        valid = False
        if self._arguments:
            # Walk through all argument conditions in chain and validate the provided arguments
            for argument in self._arguments:
                arg = argument.argument
                try:
                    self._args = {}
                    self._recursive_set(0, arg, provided_args)
                    # At this point, we found a valid argument chain
                    valid = Argument.validate_chained_args(arg)
                    # Will be true if all required args were set
                    break
                except getopt.GetoptError as err:
                    log.debug(str(err))
                    continue  # Log and try next argument in chain
                except LookupError as err:
                    log.debug(str(err))
                    continue  # Log and try next argument in chain
            if not valid:
                raise InvalidArgumentError(f"Invalid arguments [\"{', '.join(provided_args)}\"]")

    def _recursive_set(self, idx: int, argument: Argument, provided_args: List[str]) -> None:
        """ Try to set a value for each provided argument. If any failure setting occur, raise an exception to be caught
            by the parse function, so they can try the next chain in the loop.
        """
        if not argument or idx >= len(provided_args):
            return
        if not argument.set_value(provided_args[idx]):
            raise LookupError(
                f'Invalid argument "{provided_args[idx]}"')
        self._args[argument.name] = \
            provided_args[idx] if argument.next_in_chain else ','.join(provided_args[idx:])
        self._recursive_set(idx + 1, argument.next_in_chain, provided_args)

    def _shortopts(self) -> str:
        return ''.join([op.shortopt.replace('-', '') for key, op in self._options.items()])

    def _longopts(self) -> List[str]:
        return [op.longopt.replace('--', '') + ' ' for key, op in self._options.items()]



# _*_ coding: utf-8 _*_
#
# HSPyLib v0.11.1
#
# Package: main.hspylib.modules.cli.application
"""Package initialization."""

__all__ = [
  'application', 
  'argument', 
  'argument_chain', 
  'option'
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @Package: hspylib.main.hspylib.core.crud.db
      @file: db_repository.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from abc import abstractmethod
from typing import Tuple

from hspylib.core.config.app_config import AppConfigs
from hspylib.core.crud.crud_entity import CrudEntity
from hspylib.core.crud.crud_repository import CrudRepository


class DBRepository(CrudRepository):
    """TODO"""

    def __init__(self):
        super().__init__()
        self.hostname = AppConfigs.INSTANCE['datasource.hostname']
        self.port = AppConfigs.INSTANCE.get_int('datasource.port')
        self.user = AppConfigs.INSTANCE['datasource.username']
        self.password = AppConfigs.INSTANCE['datasource.password']
        self.database = AppConfigs.INSTANCE['datasource.database']

    def __str__(self):
        return "{}@{}:{}/{}".format(self.user, self.hostname, self.port, self.database)

    @abstractmethod
    def connect(self):
        """TODO"""

    @abstractmethod
    def disconnect(self):
        """TODO"""

    @abstractmethod
    def is_connected(self):
        """TODO"""

    @abstractmethod
    def execute(self, sql_statement: str, auto_commit: bool, *params):
        """TODO"""

    @abstractmethod
    def commit(self):
        """TODO"""

    @abstractmethod
    def rollback(self):
        """TODO"""

    @abstractmethod
    def row_to_entity(self, row: Tuple) -> CrudEntity:
        """TODO"""

    @abstractmethod
    def table_name(self) -> str:
        """TODO"""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.qt.promotions
      @file: htablemodel.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

import collections
import logging as log
from typing import Type, List, TypeVar, Union, Tuple

from PyQt5.QtCore import QAbstractTableModel, QModelIndex, QVariant, Qt
from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import QTableView

from hspylib.core.tools.commons import class_attribute_names, class_attribute_values

T = TypeVar('T')


class HTableModel(QAbstractTableModel):
    """TODO"""

    def __init__(
        self,
        parent: QTableView,
        clazz: Type,
        headers: List[str] = None,
        table_data: List[T] = None,
        cell_alignments: List[Qt.AlignmentFlag] = None,
        max_rows: int = 1000):

        QAbstractTableModel.__init__(self, parent)
        self.parent = parent
        self.clazz = clazz
        self.table_data = collections.deque(maxlen=max_rows)
        list(map(self.table_data.append, table_data or []))
        self.headers = headers or self._headers_by_entity()
        self.cell_alignments = cell_alignments
        self.parent.setModel(self)
        log.info('%s table_headers=%s', clazz.__class__.__name__, '|'.join(self.headers))

    def row(self, index: Union[int, QModelIndex]) -> T:
        """TODO"""
        return self.table_data[index.row() if isinstance(index, QModelIndex) else index]

    def push_data(self, data: List[T]) -> None:
        """TODO"""
        list(map(self.table_data.append, data))
        self.layoutChanged.emit()

    def clear(self):
        """TODO"""
        self.table_data.clear()
        self.layoutChanged.emit()

    def selected_rows(self) -> Tuple[List[QModelIndex], List[T]]:
        """TODO"""
        model = self.parent.selectionModel()
        rows = model.selectedRows() if model else []
        return rows, [self.table_data[r.row()] for r in rows] if model else []

    def remove_rows(self, rows: List[Union[int, QModelIndex]]):
        """TODO"""
        rows.sort(reverse=True)
        for r in rows:
            idx = r.row() if isinstance(r, QModelIndex) else r
            self.removeRow(idx)

    def removeRow(self, row: int, parent: QModelIndex = ...) -> bool:  # pylint: disable=unused-argument
        """TODO"""
        if 0 <= row < len(self.table_data):
            self.beginRemoveRows(QModelIndex(), row, row)
            del self.table_data[row]
            self.endRemoveRows()
            self.layoutChanged.emit()
            return True
        return False

    def sort(self, column: int, order: Qt.SortOrder = ...) -> None:
        """TODO"""
        keys = class_attribute_names(self.clazz)
        self.table_data = sorted(self.table_data, key=lambda x: getattr(x, keys[column]), reverse=bool(order))
        self.layoutChanged.emit()

    def data(self, index: QModelIndex, role: int = ...) -> QVariant:
        """TODO"""
        ret_val = None
        entity = class_attribute_values(self.table_data[index.row()].__dict__)[index.column()]
        str_entity = str(entity) if entity is not None else ''
        if role == Qt.DisplayRole:
            ret_val = QVariant(str_entity)
        elif role == Qt.TextAlignmentRole:
            ret_val = QVariant(self.cell_alignments[index.column()]) \
                if self.cell_alignments else QVariant(Qt.AlignLeft | Qt.AlignVCenter)
        elif role == Qt.BackgroundColorRole:
            light_gray = QColor(57, 57, 57)
            ret_val = QVariant(light_gray if index.row() % 2 == 0 else '')

        return ret_val if ret_val else QVariant()

    def headerData(self, section: int, orientation: Qt.Orientation, role: int = ...) -> QVariant:
        """TODO"""
        ret_val = None
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            ret_val = QVariant(self.headers[section].upper()) \
                if len(self.headers) >= section else QVariant('-')
        elif orientation == Qt.Vertical and role == Qt.DisplayRole:
            ret_val = QVariant(str(section))

        return ret_val if ret_val else QVariant()

    def rowCount(self, parent: QModelIndex = ...) -> int:  # pylint: disable=unused-argument
        """TODO"""
        return len(self.table_data) if self.table_data and len(self.table_data) > 0 else 0

    def columnCount(self, parent: QModelIndex = ...) -> int:  # pylint: disable=unused-argument
        """TODO"""
        return len(self.table_data[0].__dict__.keys()) if self.table_data and len(self.table_data) > 0 else 0

    def _headers_by_entity(self) -> List[str]:
        """TODO"""
        attributes = class_attribute_names(self.clazz)
        return [str(x).capitalize() for x in attributes]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.tui.extra.minput
      @file: form_field.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from typing import Any

from hspylib.modules.cli.icons.font_awesome.form_icons import FormIcons
from hspylib.modules.cli.tui.extra.minput.access_type import AccessType
from hspylib.modules.cli.tui.extra.minput.input_type import InputType
from hspylib.modules.cli.tui.extra.minput.input_validator import InputValidator


class FormField:
    """TODO"""

    def __init__(
        self,
        label: str = None,
        itype: InputType = InputType.TEXT,
        min_length: int = 0,
        max_length: int = 30,
        access_type: AccessType = AccessType.READ_WRITE,
        value: Any = '',
        validator: InputValidator = None):

        self.value = value
        self.label = label
        self.itype = itype
        self.min_length = min_length
        self.max_length = max_length
        self.access_type = access_type
        self.validator = validator or InputValidator.anything(min_length, max_length)
        self.icon = self.get_icon()

    def __str__(self) -> str:
        return f"{self.label}: {self.itype}({self.min_length}-{self.max_length}) [{self.access_type}] = '{self.value}'"

    def __repr__(self):
        return str(self)

    def get_icon(self) -> FormIcons:
        if self.access_type == AccessType.READ_ONLY:
            icon = FormIcons.LOCKED
        elif self.itype == InputType.PASSWORD:
            icon = FormIcons.HIDDEN
        elif self.itype == InputType.CHECKBOX:
            icon = FormIcons.MARKED
        elif self.itype == InputType.SELECT:
            icon = FormIcons.SELECTABLE
        elif self.itype == InputType.MASKED:
            icon = FormIcons.MASKED
        else:
            icon = FormIcons.EDITABLE

        return icon

    def can_write(self) -> bool:
        return self.access_type == AccessType.READ_WRITE

    def assign(self, value: Any) -> bool:
        if self.validate(value):
            self.value = value
            return True

        return False

    def validate(self, value: Any = None) -> bool:
        return self.validator.validate(str(value) or str(self.value)) if self.validator else False



# _*_ coding: utf-8 _*_
#
# HSPyLib v0.11.1
#
# Package: main.hspylib.core.tools
"""Package initialization."""

__all__ = [
  'commons', 
  'constants', 
  'git_tools', 
  'preconditions', 
  'text_tools', 
  'validator'
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.application
      @file: argument_chain.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from hspylib.modules.cli.application.argument import Argument


class ArgumentChain:
    """TODO"""

    @staticmethod
    class ArgumentChainBuilder:
        """TODO"""

        def __init__(self):
            self.chained_arguments = set()

        def when(self, arg_name: str, val_regex: str, required: bool = True) -> 'ArgumentChain.ChainedArgument':
            argument = Argument(arg_name, val_regex, required)
            next_in_chain = ArgumentChain.ChainedArgument(self, argument)
            self.chained_arguments.add(next_in_chain)
            return next_in_chain

        def just(self, arg_name: str, val_regex: str, required: bool = True) -> 'ArgumentChain.ArgumentChainBuilder':
            argument = Argument(arg_name, val_regex, required)
            self.chained_arguments.add(ArgumentChain.ChainedArgument(self, argument))
            return self

        def build(self) -> set:
            return self.chained_arguments

    @staticmethod
    class ChainedArgument:
        """TODO"""

        def __init__(self, parent: 'ArgumentChain.ArgumentChainBuilder', argument: Argument):
            self.parent = parent
            self.argument = argument

        def __str__(self):
            return str(self.argument)

        def accept(self, arg_name: str, val_regex) -> 'ArgumentChain.ChainedArgument':
            """TODO"""
            argument = Argument(arg_name, val_regex, False)
            next_in_chain = ArgumentChain.ChainedArgument(self.parent, argument)
            self.argument.set_next(argument)
            return next_in_chain

        def require(self, arg_name: str, val_regex) -> 'ArgumentChain.ChainedArgument':
            """TODO"""
            argument = Argument(arg_name, val_regex)
            next_in_chain = ArgumentChain.ChainedArgument(self.parent, argument)
            self.argument.set_next(argument)
            return next_in_chain

        def end(self) -> 'ArgumentChain.ArgumentChainBuilder':
            """TODO"""
            return self.parent

    @classmethod
    def builder(cls) -> 'ArgumentChain.ArgumentChainBuilder':
        """TODO"""
        return cls.ArgumentChainBuilder()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @Package: hspylib.main.hspylib.core.crud
      @file: crud_service.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from typing import Generic, List, Optional, TypeVar
from uuid import UUID

from hspylib.core.exception.exceptions import EntityNotFoundError

E = TypeVar('E')
R = TypeVar('R')


class CrudService(Generic[E, R]):
    """TODO"""

    def __init__(self, repository: R):
        self.repository = repository

    def save(self, entity: E) -> None:
        """TODO"""
        if not self.get(entity.uuid):
            self.repository.insert(entity)
        else:
            self.repository.update(entity)

    def remove(self, entity: E) -> None:
        """TODO"""
        if not self.get(entity.uuid):
            raise EntityNotFoundError(
                "{} was not found: {}".format(entity.__class__, entity))
        self.repository.delete(entity)

    def list(self, filters: str = None) -> List[E]:
        """TODO"""
        ret_val = self.repository.find_all(filters)
        return ret_val if ret_val else []

    def get(self, uuid: UUID) -> Optional[E]:
        """TODO"""
        return self.repository.find_by_id(str(uuid))



# _*_ coding: utf-8 _*_
#
# HSPyLib v0.11.1
#
# Package: main.hspylib.core.crud
"""Package initialization."""

__all__ = [
  'crud_entity', 
  'crud_repository', 
  'crud_service', 
  'db', 
  'file'
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.tui
      @file: menu_item.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from abc import ABC, abstractmethod
from typing import Optional

from hspylib.core.exception.exceptions import InputAbortedError
from hspylib.core.tools.commons import sysout
from hspylib.modules.cli.tui.menu.menu import Menu
from hspylib.modules.cli.tui.menu.menu_utils import MenuUtils


class MenuItem(Menu, ABC):
    """TODO"""

    def __init__(self, parent: Menu = None, title: str = None):
        self.title = title
        self.parent = parent
        self.done = False
        self.selected = None
        self.items = None
        self.options = None
        self.menu_data = None

    def __str__(self):
        return self.menu_data if self.menu_data else ''

    def execute(self) -> Optional[Menu]:
        """TODO"""

        while self.selected != 0 and not self.done:
            sysout(str(self))
            try:
                self.selected = MenuUtils.prompt(end='$ ')
                if not self.selected:
                    continue
                if self.selected.isalnum() and self.is_valid_option():
                    return self.trigger_menu_item()
                MenuUtils.print_error("Invalid option", self.selected)
                self.selected = None
            except InputAbortedError:
                continue

    @abstractmethod
    def trigger_menu_item(self) -> Optional[Menu]:
        """TODO"""

    def is_valid_option(self) -> bool:
        """TODO"""
        if not self.options or not self.selected:
            return False
        if self.selected.isdigit():
            return int(self.selected) in self.options

        return str(self.selected) in self.options



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.application
      @file: option.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from typing import Callable


class Option:
    """TODO"""

    def __init__(
        self,
        shortopt: chr,
        longopt: str,
        has_argument: bool = False,
        cb_handler: Callable = None):
        self.name = longopt.replace('--', '')
        self.shortopt = f"-{shortopt.replace('-', '')}{':' if has_argument > 0 else ''}"
        self.longopt = f"--{longopt.replace('--', '')}{'=' if has_argument > 0 else ''}"
        self.has_argument = has_argument
        self.cb_handler = cb_handler

    def __str__(self) -> str:
        return ("name: {}, shortopt: {}, longopt: {}, has_argument: {}, cb_handler: {}".format(
            self.name, self.shortopt, self.longopt, self.has_argument, self.cb_handler))

    def __repr__(self):
        return f"[{self.shortopt.replace('-', '').replace(':', '')}, {self.longopt.replace('--', '').replace('=', '')}]"

    def __eq__(self, other: str) -> bool:
        clean_opt = other.replace('-', '').replace(':', '').replace('=', '').strip()
        return \
            clean_opt == self.shortopt.replace('-', '').replace(':', '').strip() or \
            clean_opt == self.longopt.replace('--', '').replace('=', '').strip()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file

   @project: HSPyLib
   @package: hspylib.main.hspylib.addons.widman.widgets
      @file: widget_free.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""
import re
import threading
from time import sleep

from hspylib.addons.widman.widget import Widget
from hspylib.core.enums.exit_code import ExitCode
from hspylib.core.tools.commons import human_readable_bytes, sysout
from hspylib.modules.cli.icons.font_awesome.widget_icons import WidgetIcons
from hspylib.modules.cli.keyboard import Keyboard
from hspylib.modules.cli.vt100.terminal import Terminal


class WidgetFree(Widget):
    """HSPyLib to Report current system memory usage"""
    WIDGET_ICON = WidgetIcons.DATABASE
    WIDGET_NAME = "Free"
    TOOLTIP = "Report system memory usage."
    USAGE = "Usage: Free"
    VERSION = (0, 2, 0)

    def __init__(self):
        super().__init__(
            self.WIDGET_ICON,
            self.WIDGET_NAME,
            self.TOOLTIP,
            self.USAGE,
            self.VERSION)
        self.is_alive = True
        self.report_interval = 1.5

    def execute(self, *args) -> ExitCode:
        while not Keyboard.kbhit():
            tr = threading.Thread(target=self._report_usage)
            tr.start()
            sleep(self.report_interval)

        return ExitCode.SUCCESS

    # pylint: disable=too-many-locals
    @staticmethod
    def _report_usage():
        """Display the memory usage for the cycle"""
        ps = Terminal.shell_exec('ps -caxm -orss,comm')  # Get process info
        vm = Terminal.shell_exec('vm_stat')

        process_lines = ps.split('\n')  # Iterate processes
        sep = re.compile(' +')
        rss_total = 0  # kB

        for row in range(1, len(process_lines)):
            row_text = process_lines[row].strip()
            row_elements = sep.split(row_text)
            if re.match('^[0-9]+$', row_elements[0]):
                rss = float(row_elements[0]) * 1024
            else:
                rss = 0
            rss_total += rss

        vm_lines = vm.split('\n')  # Process vm_stat
        sep = re.compile(': +')
        vm_stats = {}

        for row in range(1, len(vm_lines) - 2):
            row_text = vm_lines[row].strip()
            row_elements = sep.split(row_text)
            vm_stats[(row_elements[0])] = int(row_elements[1].strip('\\.')) * 4096

        wired, wu = human_readable_bytes(vm_stats["Pages wired down"])
        active, au = human_readable_bytes(vm_stats["Pages active"])
        inactive, iu = human_readable_bytes(vm_stats["Pages inactive"])
        free, fu = human_readable_bytes(vm_stats["Pages free"])
        real, ru = human_readable_bytes(rss_total)  # Total memory

        sysout('%HOM%%ED2%%MOD(0)%', end='')
        sysout(f"\n%ORANGE%Reporting system memory usage:%NC% \n{'-' * 30}")
        sysout(f"    %GREEN%Wired Memory: {wired:6s} {wu:2s}")
        sysout(f"   %GREEN%Active Memory: {active:6s} {au:2s}")
        sysout(f" %GREEN%Inactive Memory: {inactive:6s} {iu:2s}")
        sysout(f"     %GREEN%Free Memory: {free:6s} {fu:2s}")
        sysout(f"     %GREEN%Real Memory: {real:6s} {ru:2s}")
        sysout('\n%YELLOW%Press [Enter] to exit ...', end='')
